/************************************************************************
 Author: Jingcao Hu  (jingcao@ece.cmu.edu)
 File name: nocmap.hpp
 Date Created:  <Wed Feb  2 02:33:07 2005>
 Last Modified: <Thu Mar 31 11:02:15 2005>
 Description: 
************************************************************************/

#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include "tile.hpp"
#include "app.hpp"
#include "util.hpp"
#include "arch.hpp"
#include "mapping.hpp"
#include "misc.hpp"
#include "msg.hpp"

using namespace std;

Param param;

pLink gLink = NULL;
pTile gTile = NULL;
vector<pProcess> gProcess;

int gProcNum;
int gLinkNum;
int gTileNum = 16;
int g_edge_size= 4;    //How many tiles per edge

int Process::proc_num = 0;
int Tile::cnt = 0;
int Tile::totalTiles = 0;
int Link::cnt = 0;
int MappingNode::cnt = 0;

int main(int argn, char** argv) {

    srand(param.seed);
    gProcNum = 16;
    gTileNum = 16;

    gProcess.clear();

    strcpy(param.exec_str, argv[0]);
    parse_options(argn, argv);

    if (param.parse_error) {
        cerr << ERRO_PARSING_OPTIONS;
        return -1;
    }

    if (param.quit_flag)
        return 0;

    //By calling initialize, classes are constructed, with some randomly
    //generated communication pattern and the default routing table. 
    //The processes parameters can be overloaded by calling parseTGFile()
    //later, and the routing table can be later be override by calling
    //parseRoutingTable()
    if (!initialize()) {
        cerr << ERRO_SYSTEM_INIT;
        return -1;
    }

    mapProcToTile(RANDOM_MAPPING);

    if (param.do_mapping) 
        optimizeMapping();

    if (param.read_mapping) 
        read_mapping_table(param.read_mapping_table_file);

    if (param.dump_mapping) 
        dump_mapping_table(param.dump_mapping_table_file);

    if (param.dump_apcg)
        dump_apcg(param.dump_apcg_file);

    if (param.dump_routing)
        dump_routing_tables(param.routing_table_file);

    analyzeIt();

    //    print_comm_statistics();

    clean();
    return 1;
}



