#!/usr/bin/perl -w
# Author: Jingcao Hu  (jingcao@ece.cmu.edu)
# File name: remap_traffic.pl
# Date Created:  <Fri Sep 24 19:04:16 2004>
# Last Modified: <Sun Jan 23 15:50:17 2005>
# Description: A simple perl script which is used to remap a traffic 
# config file.
########################################################################

use strict ;

my $numArgs = $#ARGV + 1;

if ($numArgs != 3) {
    print ("Usage: \nremap_traffic.pl orig_traffic_config map_file new_traffic_config\n");
    exit;
}

my $old_config_file_name = $ARGV[0];
my $map_file_name = $ARGV[1];
my $new_config_file_name = $ARGV[2];

open(old_config_file, $old_config_file_name) ||
    die("Unable to open file " . $old_config_file_name . " for read.\n");

open(map_file, $map_file_name) ||
    die("Unable to open file " . $map_file_name . " for read.\n");

open(new_config_file, ">$new_config_file_name") ||
    die("Unable to open file " . $new_config_file_name . " for write.\n"); 

# map_file is in the format of (for each line)
# old_id     mapped_id
my @new_id = ( 0 );
my $count = 0;

while (my $input_line = <map_file>) {
    if ( $input_line =~ /^\#/ ) {
        next;
    }
    my @id = split(/[ \t]+/, $input_line);
    if ($#id + 1 != 2) {
        die ("Unknown map file format\n");
    }
    my $max = $id[0];
    if ( $max < $id[1] ) {
        $max = $id[1];
    }
    chomp($id[1]);
    if ($max > $#new_id) {
# we need to expand the array
        for ($count = 0; $count <= $max - $#new_id; $count ++) {
            @new_id = (@new_id, -1);
        }
    }
    $new_id[$id[0]] = $id[1];
}

close(map_file);


# now starts remapping, add some heading here

print new_config_file ("# File generated by remapping utility remap_config.pl:\n");
print new_config_file ("# remap_traffic.pl $old_config_file_name $map_file_name $new_config_file_name\n");

while (my $input_line = <old_config_file>) {
    if ( $input_line =~ /^\#/ ) { # if this is a comment line
        next;
    }
    else {
        if ( $input_line =~ /^\@NODE/ ) {
            my @list = split(/[ \t]+/, $input_line);
            (my $node, my $old_id) = @list;
            print new_config_file ("\@NODE\t", $new_id[$old_id], "\n");
        }
        elsif ( $input_line =~ /^packet_to_destination_rate/ ) {
            my @list = split(/[ \t]+/, $input_line);
            (my $semantics, my $old_id, my $rate) = @list;
            print new_config_file ($semantics, "\t", $new_id[$old_id], "\t", $rate);
        }
        else {
            if ( !($input_line =~ /^\n/) ) {
                print ("Unknown semantics\n" . $input_line);
            }
        }
    }
}

close(new_config_file);
close(old_config_file);

